/*
* JBoss, Home of Professional Open Source
* Copyright 2005, JBoss Inc., and individual contributors as indicated
* by the @authors tag. See the copyright.txt in the distribution for a
* full listing of individual contributors.
*
* This is free software; you can redistribute it and/or modify it
* under the terms of the GNU Lesser General Public License as
* published by the Free Software Foundation; either version 2.1 of
* the License, or (at your option) any later version.
*
* This software is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this software; if not, write to the Free
* Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
* 02110-1301 USA, or see the FSF site: http://www.fsf.org.
*/
package org.jboss.webservice.jaf;

import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.UnsupportedFlavorException;
import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.OutputStream;

import javax.activation.ActivationDataFlavor;
import javax.activation.DataContentHandler;
import javax.activation.DataSource;
import javax.mail.internet.ContentType;
import javax.xml.transform.Source;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.sax.SAXSource;
import javax.xml.transform.stream.StreamSource;

import org.dom4j.Document;
import org.dom4j.io.DOMReader;
import org.dom4j.io.SAXReader;
import org.dom4j.io.XMLWriter;
import org.xml.sax.EntityResolver;
import org.xml.sax.InputSource;

/**
 * A JAF DataContentHandler for text/xml and application/xml that <p/> supports
 * the javax.xml.transform.Source type required by SAAJ
 *
 * @author Scott.Stark@jboss.org
 * @version $Revision: 1.3.4.2 $
 */

public class XMLDataContentHandler implements DataContentHandler

{

   static NullEntityResolver resolver = new NullEntityResolver();

   public Object getContent(DataSource dataSource) throws IOException
   {
      StreamSource stream = new StreamSource(dataSource.getInputStream());
      return stream;
   }

   public Object getTransferData(DataFlavor dataFlavor, DataSource dataSource) throws UnsupportedFlavorException, IOException
   {
      Object data = null;

      // Handle text/xml & application/xml
      String mimeType = dataFlavor.getMimeType();
      if (mimeType.startsWith("text/xml") || mimeType.startsWith("application/xml"))
      {
         data = new StreamSource(dataSource.getInputStream());
      }

      return data;
   }

   public DataFlavor[] getTransferDataFlavors()
   {
      DataFlavor flavors[] = { new ActivationDataFlavor(Source.class, "text/xml", "XML"),
            new ActivationDataFlavor(Source.class, "application/xml", "XML") };

      return flavors;
   }

   public void writeTo(Object obj, String mimeType, OutputStream os) throws IOException
   {
      if (mimeType.startsWith("text/xml") == false && mimeType.startsWith("application/xml") == false)
      {
         String msg = "Only text/xml, application/xml are supported, mimeType=" + mimeType;

         throw new IOException(msg);
      }

      try
      {
         if (obj instanceof StreamSource)
         {
            StreamSource ss = (StreamSource)obj;
            XMLWriter writer = new XMLWriter(os);
            SAXReader reader = new SAXReader(false);
            reader.setIncludeExternalDTDDeclarations(false);
            reader.setEntityResolver(resolver);
            Document doc = reader.read(ss.getInputStream());
            writer.write(doc);
         }
         else if (obj instanceof DOMSource)
         {
            DOMSource ds = (DOMSource)obj;
            org.w3c.dom.Document doc = ds.getNode().getOwnerDocument();
            DOMReader reader = new DOMReader();
            Document doc2 = reader.read(doc);
            XMLWriter writer = new XMLWriter(os);
            writer.write(doc2);
         }
         else if (obj instanceof SAXSource)
         {
            SAXSource ss = (SAXSource)obj;
            XMLWriter writer = new XMLWriter(os);
            SAXReader reader = new SAXReader(false);
            reader.setIncludeExternalDTDDeclarations(false);
            reader.setEntityResolver(resolver);
            Document doc = reader.read(ss.getInputSource());
            writer.write(doc);
         }
         else if (obj instanceof String)
         {
            String charset = new ContentType(mimeType).getParameter("charset");
            if (charset == null)
               charset = "utf-8";

            os.write(obj.toString().getBytes(charset));
         }
      }
      catch (Exception e)
      {
         if (e instanceof IOException)
            throw (IOException)e;
         else
         {
            IOException ioe = new IOException("Failed to write xml");
            ioe.initCause(e);
         }
      }
   }

   static class NullEntityResolver implements EntityResolver
   {
      static String dtd = "";

      static ByteArrayInputStream bais = new ByteArrayInputStream(dtd.getBytes());

      public InputSource resolveEntity(String publicId, String systemId)
      {
         return new InputSource(bais);
      }
   }
}
