/*
* JBoss, Home of Professional Open Source
* Copyright 2005, JBoss Inc., and individual contributors as indicated
* by the @authors tag. See the copyright.txt in the distribution for a
* full listing of individual contributors.
*
* This is free software; you can redistribute it and/or modify it
* under the terms of the GNU Lesser General Public License as
* published by the Free Software Foundation; either version 2.1 of
* the License, or (at your option) any later version.
*
* This software is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this software; if not, write to the Free
* Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
* 02110-1301 USA, or see the FSF site: http://www.fsf.org.
*/
package org.jboss.webservice.metadata;

import java.util.ArrayList;
import java.util.HashMap;
import java.net.URLClassLoader;

// $Id: WebservicesMetaData.java,v 1.7.4.1 2005/10/29 05:07:17 starksm Exp $

/**
 * XML Binding root element for <code>webservices.xml</code>
 *
 * @author Thomas.Diesler@jboss.org
 * @since 15-April-2004
 */
public class WebservicesMetaData
{
   // The required <webservice-description> elements
   private ArrayList webserviceDescriptions = new ArrayList();

   // The modules resource class loader
   private URLClassLoader resourceCL;

   // Maps the wsdl-file to the wsdl Document
   // Note the same wsdl can be used in multiple webservice descriptions
   HashMap wsdlFileMap = new HashMap();
   // Maps the jaxrpc-mapping-file to {@link JavaWsdlMapping} object
   // Note the same jaxrpc-mapping.xml can be used in multiple webservice descriptions
   HashMap jaxrpcMappingFileMap = new HashMap();

   /**
    * Construct webservices meta data, with a given resource class loader.
    */
   public WebservicesMetaData(URLClassLoader resourceCL)
   {
      this.resourceCL = resourceCL;
   }

   public URLClassLoader getResourceLoader()
   {
      return resourceCL;
   }

   public void addWebserviceDescription(WebserviceDescriptionMetaData webserviceDescription)
   {
      webserviceDescriptions.add(webserviceDescription);
   }

   public WebserviceDescriptionMetaData[] getWebserviceDescriptions()
   {
      WebserviceDescriptionMetaData[] array = new WebserviceDescriptionMetaData[webserviceDescriptions.size()];
      webserviceDescriptions.toArray(array);
      return array;
   }
}
